@@ -10,39 +10,26 @@
 %                        F      IIIII    T    SSSSS                           %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read/Write Flexible Image Transport System Images.               %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2001 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,8 +39,20 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "defines.h"
+#include "magick/studio.h"
+#include "magick/attribute.h"
+#include "magick/blob.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/error.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
@@ -72,17 +71,16 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsFITS returns True if the image format type, identified by the
+%  IsFITS() returns True if the image format type, identified by the
 %  magick string, is FITS.
 %
 %  The format of the IsFITS method is:
 %
-%      unsigned int IsFITS(const unsigned char *magick,
-%        const unsigned int length)
+%      unsigned int IsFITS(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFITS returns True if the image format type is FITS.
+%    o status:  Method IsFITS() returns True if the image format type is FITS.
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
@@ -91,8 +89,7 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFITS(const unsigned char *magick,
-  const unsigned int length)
+static unsigned int IsFITS(const unsigned char *magick,const size_t length)
 {
   if (length < 6)
     return(False);
@@ -114,7 +111,7 @@ static unsigned int IsFITS(const unsigned char *magick,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFITSImage reads a FITS image file and returns it.  It
+%  ReadFITSImage() reads a FITS image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -140,13 +137,9 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 {
   typedef struct _FITSInfo
   {
-    unsigned int
-      simple;
-
     int
-      bits_per_pixel;
-
-    unsigned int
+      simple,
+      bits_per_pixel,
       columns,
       rows,
       number_axes,
@@ -160,11 +153,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   } FITSInfo;
 
   char
-    long_quantum[8],
     keyword[MaxTextExtent],
     value[MaxTextExtent];
 
   double
+    exponential[2048],
     pixel,
     scale,
     scaled_pixel;
@@ -180,42 +173,63 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 
   int
     c,
-    j,
-    packet_size,
-    y;
+    quantum;
 
   long
-    count,
-    quantum;
+    exponent,
+    j,
+    k,
+    l,
+    scene,
+    y;
+
+  MagickSizeType
+    number_pixels;
 
   register IndexPacket
     *indexes;
 
-  register int
-    i,
+  register long
     x;
 
   register PixelPacket
     *q;
 
+  register long
+    i;
+
   register unsigned char
     *p;
 
+  ssize_t
+    count;
+
+  size_t
+    packet_size;
+
   unsigned char
-    *fits_pixels;
+    *fits_pixels,
+    long_quantum[8];
 
   unsigned int
-    scene,
     status,
     value_expected;
-
   /*
     Open image file.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize image header.
   */
@@ -232,7 +246,6 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     Decode image header.
   */
   c=ReadBlobByte(image);
-  count=1;
   if (c == EOF)
     {
       DestroyImage(image);
@@ -240,11 +253,8 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
     }
   for ( ; ; )
   {
-    if (!isalnum(c))
-      {
-        c=ReadBlobByte(image);
-        count++;
-      }
+    if (isalnum((int) ((unsigned char) c)) == 0)
+      c=ReadBlobByte(image);
     else
       {
         register char
@@ -256,48 +266,47 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=keyword;
         do
         {
-          if ((p-keyword) < (MaxTextExtent-1))
-            *p++=c;
+          if ((size_t) (p-keyword) < (MaxTextExtent-1))
+            *p++=(char) c;
           c=ReadBlobByte(image);
-          count++;
-        } while (isalnum(c) || (c == '_'));
+        } while ((isalnum((int) ((unsigned char) c)) != False) ||
+                 ((int) c == '_'));
         *p='\0';
         if (LocaleCompare(keyword,"END") == 0)
           break;
         value_expected=False;
-        while (isspace(c) || (c == '='))
+        while ((isspace((int) ((unsigned char) c)) != False) ||
+               ((int) c == '='))
         {
-          if (c == '=')
+          if ((int) c == '=')
             value_expected=True;
           c=ReadBlobByte(image);
-          count++;
         }
         if (value_expected == False)
           continue;
         p=value;
         while (isalnum(c) || (c == '-') || (c == '+') || (c == '.'))
         {
-          if ((p-value) < (MaxTextExtent-1))
+          if ((size_t) (p-value) < (MaxTextExtent-1))
             *p++=c;
           c=ReadBlobByte(image);
-          count++;
         }
         *p='\0';
         /*
           Assign a value to the specified keyword.
         */
         if (LocaleCompare(keyword,"SIMPLE") == 0)
-          fits_info.simple=(*value == 'T') || (*value == 't');
+          fits_info.simple=(int) ((*value == 'T') || (*value == 't'));
         if (LocaleCompare(keyword,"BITPIX") == 0)
-          fits_info.bits_per_pixel=(unsigned int) atoi(value);
+          fits_info.bits_per_pixel=atoi(value);
         if (LocaleCompare(keyword,"NAXIS") == 0)
-          fits_info.number_axes=(unsigned int) atoi(value);
+          fits_info.number_axes=atoi(value);
         if (LocaleCompare(keyword,"NAXIS1") == 0)
-          fits_info.columns=(unsigned int) atoi(value);
+          fits_info.columns=atoi(value);
         if (LocaleCompare(keyword,"NAXIS2") == 0)
-          fits_info.rows=(unsigned int) atoi(value);
+          fits_info.rows=atoi(value);
         if (LocaleCompare(keyword,"NAXIS3") == 0)
-          fits_info.number_scenes=(unsigned int) atoi(value);
+          fits_info.number_scenes=atoi(value);
         if (LocaleCompare(keyword,"DATAMAX") == 0)
           fits_info.max_data=atof(value);
         if (LocaleCompare(keyword,"DATAMIN") == 0)
@@ -306,60 +315,73 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
           fits_info.zero=atof(value);
         if (LocaleCompare(keyword,"BSCALE") == 0)
           fits_info.scale=atof(value);
+        (void) SetImageAttribute(image,keyword,value);
       }
-    while (isspace(c))
-    {
+    while ((TellBlob(image) % 80) != 0)
       c=ReadBlobByte(image);
-      count++;
-    }
+    c=ReadBlobByte(image);
   }
-  while (count > 2880)
-    count-=2880;
-  for ( ; count < 2880; count++)
-    (void) ReadBlobByte(image);
+  while ((TellBlob(image) % 2880) != 0)
+    c=ReadBlobByte(image);
   /*
     Verify that required image information is defined.
   */
-  if ((!fits_info.simple) || (fits_info.number_axes < 1) ||
-      (fits_info.number_axes > 4) ||
-      (fits_info.columns*fits_info.rows) == 0)
-    ThrowReaderException(CorruptImageWarning,"image type not supported",image);
-  for (scene=0; scene < fits_info.number_scenes; scene++)
+  number_pixels=(MagickSizeType) fits_info.columns*fits_info.rows;
+  if ((fits_info.simple == 0) || (fits_info.number_axes < 1) ||
+      (fits_info.number_axes > 4) || (number_pixels == 0))
+    ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
+  if (fits_info.bits_per_pixel == -32)
+    {
+      exponential[150]=1.0;
+      for (i=151; i < 256; i++)
+        exponential[i]=2.0*exponential[i-1];
+      for (i=149; i >= 0; i--)
+        exponential[i]=exponential[i+1]/2.0;
+    }
+  if (fits_info.bits_per_pixel == -64)
+    {
+      exponential[1075]=1.0;
+      for (i=1076; i < 2048; i++)
+        exponential[i]=2.0*exponential[i-1];
+      for (i=1074; i >= 0; i--)
+        exponential[i]=exponential[i+1]/2.0;
+    }
+  for (scene=0; scene < (long) fits_info.number_scenes; scene++)
   {
     /*
       Create linear colormap.
     */
-    image->columns=fits_info.columns;
-    image->rows=fits_info.rows;
-    image->depth=fits_info.bits_per_pixel <= 8 ? 8 : QuantumDepth;
+    image->columns=(unsigned long) fits_info.columns;
+    image->rows=(unsigned long) fits_info.rows;
+    image->depth=fits_info.bits_per_pixel <= 8 ? 8UL : QuantumDepth;
     image->storage_class=PseudoClass;
-    image->scene=scene;
-    if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(FileOpenWarning,"Unable to open file",image);
-    if (image_info->ping)
-      {
-        CloseBlob(image);
-        return(image);
-      }
+    image->scene=(unsigned long) scene;
+    if (AllocateImageColormap(image,1 << image->depth) == False)
+      ThrowReaderException(ResourceLimitError,"UnableToAllocateColormap");
+    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+        break;
     /*
       Initialize image structure.
     */
-    packet_size=fits_info.bits_per_pixel/8;
-    if (packet_size < 0)
-      packet_size=(-packet_size);
+    packet_size=(size_t) fits_info.bits_per_pixel/8;
+    if (fits_info.bits_per_pixel < 0)
+      packet_size=(size_t) (-fits_info.bits_per_pixel/8);
+    number_pixels=(MagickSizeType) image->columns*image->rows;
+    if ((packet_size*number_pixels) !=
+        (MagickSizeType) ((size_t) (packet_size*number_pixels)))
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     fits_pixels=(unsigned char *)
-      AcquireMemory(packet_size*image->columns*image->rows);
+      AcquireMagickMemory(packet_size*image->columns*image->rows);
     if (fits_pixels == (unsigned char *) NULL)
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     /*
       Convert FITS pixels to pixel packets.
     */
-    status=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
-    if (status == False)
-      ThrowReaderException(CorruptImageWarning,
-        "Insufficient image data in file",image);
-    if ((fits_info.min_data == 0.0) && (fits_info.max_data == 0.0))
+    count=ReadBlob(image,packet_size*image->columns*image->rows,fits_pixels);
+    if (count == 0)
+      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+    if ((fits_info.min_data == 0.0) || (fits_info.max_data == 0.0))
       {
         /*
           Determine minimum and maximum intensity.
@@ -367,38 +389,88 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         p=fits_pixels;
         long_quantum[0]=(*p);
         quantum=(*p++);
-        for (j=0; j < (packet_size-1); j++)
+        for (j=1; j <= (long) (packet_size-1); j++)
         {
-          long_quantum[j+1]=(*p);
+          long_quantum[j]=(*p);
           quantum=(quantum << 8) | (*p++);
         }
         pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536;
         if (fits_info.bits_per_pixel == -32)
-          pixel=(double) (*((float *) &quantum));
+          {
+            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+               (long) long_quantum[3];
+            k=(int) *long_quantum;
+            exponent=((k & 127) << 1) | (j >> 23);
+            *(float *) long_quantum=
+              exponential[exponent]*(float) (j | 0x800000);
+            if ((exponent | j) == 0)
+              *(float *) long_quantum=0.0;
+            if (k & 128)
+              *(float *) long_quantum=(-(*(float *) long_quantum));
+            pixel=(double) (*((float *) long_quantum));
+          }
         if (fits_info.bits_per_pixel == -64)
-          pixel=(double) (*((double *) long_quantum));
+          {
+            j=((long) long_quantum[1] << 24) |
+              ((long) long_quantum[2] << 16) |
+              ((long) long_quantum[3] << 8) |
+               (long) long_quantum[4];
+            k=(int) *long_quantum;
+            l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
+               (int) long_quantum[7];
+            exponent=((k & 127) << 4) | (j >> 28);
+            *(double *) long_quantum=exponential[exponent]*(16777216.0*
+              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
+            if ((exponent | j | l) == 0)
+              *(double *) long_quantum=0.0;
+            if (k & 128)
+              *(double *)long_quantum=(-(*(double *) long_quantum));
+            pixel=(double) (*((double *) long_quantum));
+          }
         fits_info.min_data=pixel*fits_info.scale-fits_info.zero;
         fits_info.max_data=pixel*fits_info.scale-fits_info.zero;
-        for (i=1; i < (int) (image->columns*image->rows); i++)
+        for (i=1; i < (long) number_pixels; i++)
         {
           long_quantum[0]=(*p);
           quantum=(*p++);
-          for (j=0; j < (packet_size-1); j++)
+          for (j=1; j <= (long) (packet_size-1); j++)
           {
-            long_quantum[j+1]=(*p);
+            long_quantum[j]=(*p);
             quantum=(quantum << 8) | (*p++);
           }
           pixel=(double) quantum;
-          if (fits_info.bits_per_pixel == 16)
-            if (pixel > 32767)
-              pixel-=65536;
           if (fits_info.bits_per_pixel == -32)
-            pixel=(double) (*((float *) &quantum));
+            {
+              j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+                 (long) long_quantum[3];
+              k=(int) *long_quantum;
+              exponent=((k & 127) << 1) | (j >> 23);
+              *(float *) long_quantum=
+                exponential[exponent]*(float) (j | 0x800000);
+              if ((exponent | j) == 0)
+                *(float *) long_quantum=0.0;
+              if (k & 128)
+                *(float *) long_quantum=(-(*(float *) long_quantum));
+              pixel=(double) (*((float *) long_quantum));
+            }
           if (fits_info.bits_per_pixel == -64)
-            pixel=(double) (*((double *) long_quantum));
+            {
+              j=((long) long_quantum[1] << 24) |
+                ((long) long_quantum[2] << 16) |
+                ((long) long_quantum[3] << 8) |
+                 (long) long_quantum[4];
+              k=(int) *long_quantum;
+              l=((int) long_quantum[5] << 16) | ((int) long_quantum[6] << 8) |
+                 (int) long_quantum[7];
+              exponent=((k & 127) << 4) | (j >> 28);
+              *(double *) long_quantum=exponential[exponent]*(16777216.0*
+                (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
+              if ((exponent | j | l) == 0)
+                *(double *) long_quantum=0.0;
+              if (k & 128)
+                *(double *)long_quantum=(-(*(double *) long_quantum));
+              pixel=(double) (*((double *) long_quantum));
+            }
           scaled_pixel=pixel*fits_info.scale-fits_info.zero;
           if (scaled_pixel < fits_info.min_data)
             fits_info.min_data=scaled_pixel;
@@ -410,57 +482,89 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
       Convert FITS pixels to pixel packets.
     */
     scale=1.0;
-    if (((fits_info.max_data-fits_info.min_data) <= 1.0) ||
-        ((fits_info.max_data-fits_info.min_data) > ((1 << image->depth)-1)))
-      scale=((1 << image->depth)-1)/(fits_info.max_data-fits_info.min_data);
+    if ((fits_info.bits_per_pixel < 0) || ((fits_info.max_data-
+        fits_info.min_data) > (double) ((1UL << image->depth)-1)))
+      scale=(double) ((1 << image->depth)-1)/
+        (fits_info.max_data-fits_info.min_data);
     p=fits_pixels;
-    for (y=image->rows-1; y >= 0; y--)
+    for (y=(long) image->rows-1; y >= 0; y--)
     {
       q=SetImagePixels(image,0,y,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (int) image->columns; x++)
+      for (x=0; x < (long) image->columns; x++)
       {
         long_quantum[0]=(*p);
         quantum=(*p++);
-        for (j=0; j < (packet_size-1); j++)
+        for (j=1; j <= (long) (packet_size-1); j++)
         {
-          long_quantum[j+1]=(*p);
+          long_quantum[j]=(*p);
           quantum=(quantum << 8) | (*p++);
         }
         pixel=(double) quantum;
-        if (fits_info.bits_per_pixel == 16)
-          if (pixel > 32767)
-            pixel-=65536;
         if (fits_info.bits_per_pixel == -32)
-          pixel=(double) (*((float *) &quantum));
+          {
+            j=((long) long_quantum[1] << 16) | ((long) long_quantum[2] << 8) |
+               (long) long_quantum[3];
+            k=(int) *long_quantum;
+            exponent=((k & 127) << 1) | (j >> 23);
+            *(float *) long_quantum=
+              exponential[exponent]*(float) (j | 0x800000);
+            if ((exponent | j) == 0)
+              *(float *) long_quantum=0.0;
+            if (k & 128)
+              *(float *) long_quantum=(-(*(float *) long_quantum));
+            pixel=(double) (*((float *) long_quantum));
+          }
         if (fits_info.bits_per_pixel == -64)
-          pixel=(double) (*((double *) long_quantum));
+          {
+            j=((long) long_quantum[1] << 24) |
+              ((long) long_quantum[2] << 16) |
+              ((long) long_quantum[3] << 8) |
+               (long) long_quantum[4];
+            k=(long) *long_quantum;
+            l=((long) long_quantum[5] << 16) | ((long) long_quantum[6] << 8) |
+               (long) long_quantum[7];
+            exponent=(long) ((((unsigned long) k & 127) << 4) |
+              ((unsigned long) j >> 28));
+            *(double *) long_quantum=exponential[exponent]*(16777216.0*
+              (double) ((j & 0x0FFFFFFF) | 0x10000000)+(double) l);
+            if ((exponent | j | l) == 0)
+              *(double *) long_quantum=0.0;
+            if ((k & 128) != 0)
+              *(double *)long_quantum=(-(*(double *) long_quantum));
+            pixel=(double) (*((double *) long_quantum));
+          }
         scaled_pixel=scale*
           (pixel*fits_info.scale-fits_info.min_data-fits_info.zero);
-        index=(IndexPacket) ((scaled_pixel < 0) ? 0 :
-          (scaled_pixel > ((1 << image->depth)-1)) ? ((1 << image->depth)-1) :
-          scaled_pixel+0.5);
-        index=ValidateColormapIndex(image,index);
+        index=(IndexPacket) ((scaled_pixel < 0.0) ? 0 :
+          ((unsigned long) scaled_pixel > ((1UL << image->depth)-1)) ?
+          ((1UL << image->depth)-1) : (unsigned long) (scaled_pixel+0.5));
+        index=ConstrainColormapIndex(image,index);
         indexes[x]=index;
         *q++=image->colormap[index];
       }
-      if (!SyncImagePixels(image))
+      if (SyncImagePixels(image) == False)
         break;
       if (QuantumTick(y,image->rows))
-        MagickMonitor(LoadImageText,y,image->rows);
+        if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
+          break;
     }
-    LiberateMemory((void **) &fits_pixels);
+    fits_pixels=(unsigned char *) RelinquishMagickMemory(fits_pixels);
+    if (EOFBlob(image) != False)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
-    if (EOFBlob(image))
-      break;
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if (scene < (fits_info.number_scenes-1))
+    if (scene < (long) (fits_info.number_scenes-1))
       {
         /*
           Allocate next image structure.
@@ -468,17 +572,18 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImages(image);
+            DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),image->filesize);
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   }
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -492,7 +597,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFITSImage adds attributes for the FITS image format to
+%  RegisterFITSImage() adds attributes for the FITS image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -510,13 +615,13 @@ ModuleExport void RegisterFITSImage(void)
     *entry;
 
   entry=SetMagickInfo("FITS");
-  entry->decoder=ReadFITSImage;
-  entry->encoder=WriteFITSImage;
-  entry->magick=IsFITS;
+  entry->decoder=(DecoderHandler *) ReadFITSImage;
+  entry->encoder=(EncoderHandler *) WriteFITSImage;
+  entry->magick=(MagickHandler *) IsFITS;
   entry->adjoin=False;
-  entry->description=AllocateString("Flexible Image Transport System");
-  entry->module=AllocateString("FITS");
-  RegisterMagickInfo(entry);
+  entry->description=AcquireString("Flexible Image Transport System");
+  entry->module=AcquireString("FITS");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -530,7 +635,7 @@ ModuleExport void RegisterFITSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFITSImage removes format registrations made by the
+%  UnregisterFITSImage() removes format registrations made by the
 %  FITS module from the list of supported formats.
 %
 %  The format of the UnregisterFITSImage method is:
@@ -540,7 +645,7 @@ ModuleExport void RegisterFITSImage(void)
 */
 ModuleExport void UnregisterFITSImage(void)
 {
-  UnregisterMagickInfo("FITS");
+  (void) UnregisterMagickInfo("FITS");
 }
 
 /*
@@ -554,7 +659,7 @@ ModuleExport void UnregisterFITSImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteFITSImage writes a Flexible Image Transport System image to a
+%  WriteFITSImage() writes a Flexible Image Transport System image to a
 %  file as gray scale intensities [0..255].
 %
 %  The format of the WriteFITSImage method is:
@@ -567,9 +672,9 @@ ModuleExport void UnregisterFITSImage(void)
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
@@ -579,76 +684,93 @@ static unsigned int WriteFITSImage(const ImageInfo *image_info,Image *image)
     buffer[81],
     *fits_info;
 
-  int
+  long
     y;
 
-  register int
+  register const PixelPacket
+    *p;
+
+  register long
     i;
 
-  register PixelPacket
-    *p;
+  size_t
+    packet_size;
 
   unsigned char
     *pixels;
 
   unsigned int
-    packet_size,
     status;
 
   /*
     Open output image file.
   */
-  status=OpenBlob(image_info,image,WriteBinaryType);
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  TransformRGBImage(image,RGBColorspace);
+    return(status);
+  (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
-  packet_size=image->depth > 8 ? 2 : 1;
-  fits_info=(char *) AcquireMemory(2880);
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
+  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  fits_info=(char *) AcquireMagickMemory(2880);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if ((fits_info == (char *) NULL) || (pixels == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Initialize image header.
   */
   for (i=0; i < 2880; i++)
     fits_info[i]=' ';
   (void) strcpy(buffer,"SIMPLE  =                    T");
-  (void) strncpy(fits_info+0,buffer,Extent(buffer));
-  FormatString(buffer,"BITPIX  =                    %d",image->depth);
-  (void) strncpy(fits_info+80,buffer,Extent(buffer));
+  (void) strncpy(fits_info+0,buffer,strlen(buffer));
+  (void) FormatMagickString(buffer,MaxTextExtent,
+    "BITPIX  =                    %ld",image->depth);
+  (void) strncpy(fits_info+80,buffer,strlen(buffer));
   (void) strcpy(buffer,"NAXIS   =                    2");
-  (void) strncpy(fits_info+160,buffer,Extent(buffer));
-  FormatString(buffer,"NAXIS1  =           %10u",image->columns);
-  (void) strncpy(fits_info+240,buffer,Extent(buffer));
-  FormatString(buffer,"NAXIS2  =           %10u",image->rows);
-  (void) strncpy(fits_info+320,buffer,Extent(buffer));
-  FormatString(buffer,"DATAMIN =           %10u",0);
-  (void) strncpy(fits_info+400,buffer,Extent(buffer));
-  FormatString(buffer,"DATAMAX =           %10u",(1 << image->depth)-1);
-  (void) strncpy(fits_info+480,buffer,Extent(buffer));
+  (void) strncpy(fits_info+160,buffer,strlen(buffer));
+  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS1  =           %10lu",
+    image->columns);
+  (void) strncpy(fits_info+240,buffer,strlen(buffer));
+  (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS2  =           %10lu",
+    image->rows);
+  (void) strncpy(fits_info+320,buffer,strlen(buffer));
+  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMIN =           %10u",0);
+  (void) strncpy(fits_info+400,buffer,strlen(buffer));
+  (void) FormatMagickString(buffer,MaxTextExtent,"DATAMAX =           %10u",
+    (1 << image->depth)-1);
+  (void) strncpy(fits_info+480,buffer,strlen(buffer));
   (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
-  (void) strncpy(fits_info+560,buffer,Extent(buffer));
+  (void) strncpy(fits_info+560,buffer,strlen(buffer));
   (void) strcpy(buffer,"END");
-  (void) strncpy(fits_info+640,buffer,Extent(buffer));
-  (void) WriteBlob(image,2880,(char *) fits_info);
-  LiberateMemory((void **) &fits_info);
+  (void) strncpy(fits_info+640,buffer,strlen(buffer));
+  (void) WriteBlob(image,2880,(unsigned char *) fits_info);
+  fits_info=(char *) RelinquishMagickMemory(fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
-  for (y=image->rows-1; y >= 0; y--)
+  for (y=(long) image->rows-1; y >= 0; y--)
   {
-    p=GetImagePixels(image,0,y,image->columns,1);
-    if (p == (PixelPacket *) NULL)
+    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
     (void) PopImagePixels(image,GrayQuantum,pixels);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
     if (QuantumTick(image->rows-y-1,image->rows))
-      MagickMonitor(SaveImageText,image->rows-y-1,image->rows);
+      {
+        status=MagickMonitor(SaveImageTag,(MagickOffsetType) image->rows-y-1,
+          image->rows,&image->exception);
+        if (status == False)
+          break;
+      }
   }
-  LiberateMemory((void **) &pixels);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(True);
 }
