@@ -10,39 +10,26 @@
 %                            F      P      X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write FlashPIX Image Format.                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2001 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,21 +39,75 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "defines.h"
+#include "magick/studio.h"
+#include "magick/attribute.h"
+#include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/error.h"
+#include "magick/geometry.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 #if defined(HasFPX)
-#if !defined(vms) && !defined(macintosh) && !defined(WIN32)
+#if !defined(vms) && !defined(macintosh) && !defined(__WINDOWS__)
 #include <fpxlib.h>
 #else
 #include "Fpxlib.h"
 #endif
 #endif
 
+#if defined(HasFPX)
 /*
   Forward declarations.
 */
 static unsigned int
   WriteFPXImage(const ImageInfo *,Image *);
+#endif
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s F P X                                                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsFPX() returns True if the image format type, identified by the
+%  magick string, is FPX.
+%
+%  The format of the IsFPX method is:
+%
+%      unsigned int IsFPX(const unsigned char *magick,const size_t length)
+%
+%  A description of each parameter follows:
+%
+%    o status:  Method IsFPX() returns True if the image format type is FPX.
+%
+%    o magick: This string is generally the first few bytes of an image file
+%      or blob.
+%
+%    o length: Specifies the length of the magick string.
+%
+%
+*/
+static unsigned int IsFPX(const unsigned char *magick,const size_t length)
+{
+  if (length < 4)
+    return(False);
+  if (memcmp(magick,"\320\317\021\340",4) == 0)
+    return(True);
+  return(False);
+}
 
 #if defined(HasFPX)
 /*
@@ -80,7 +121,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFPXImage reads a FlashPix image file and returns it.  It
+%  ReadFPXImage() reads a FlashPix image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.  This method was contributed by BillR@corbis.com.
 %
@@ -94,7 +135,7 @@ static unsigned int
 %      reading. A null image is returned if there is a memory shortage or if
 %      the image cannot be read.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -129,13 +170,13 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  int
+  long
     y;
 
   register IndexPacket
     *indexes;
 
-  register int
+  register long
     i,
     x;
 
@@ -153,7 +194,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned int
     status,
-    subimage;
+    scene;
 
   unsigned long
     height,
@@ -165,44 +206,33 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Open image.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
-  if ((image->file == stdin) || image->pipe)
     {
-      FILE
-        *file;
-
-      int
-        c;
-
-      /*
-        Copy standard input or pipe to temporary file.
-      */
-      TemporaryFilename((char *) image_info->filename);
-      file=fopen(image_info->filename,WriteBinaryType);
-      if (file == (FILE *) NULL)
-        ThrowReaderException(FileOpenWarning,"Unable to write file",image);
-      c=ReadBlobByte(image);
-      while (c != EOF)
-      {
-        (void) fputc(c,file);
-        c=ReadBlobByte(image);
-      }
-      (void) fclose(file);
-      (void) strcpy(image->filename,image_info->filename);
-      image->temporary=True;
+      DestroyImageList(image);
+      return((Image *) NULL);
     }
   CloseBlob(image);
   /*
     Initialize FPX toolkit.
   */
+  fpx_status=FPX_InitSystem();
+  if (fpx_status != FPX_OK)
+    ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowReaderException(DelegateWarning,"Unable to initialize FPX library",
-      image);
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CoderError,"UnableToInitializeFPXLibrary");
+    }
   tile_width=64;
   tile_height=64;
   flashpix=(FPXImageHandle *) NULL;
@@ -212,22 +242,29 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       fsspec;
 
     FilenameToFSSpec(image->filename,&fsspec);
-    fpx_status=FPX_OpenImageByFilename((const FSSpec &) fsspec,
+    fpx_status=FPX_OpenImageByFilename((const FSSpec &) fsspec,(char *) NULL,
 #else
-    fpx_status=FPX_OpenImageByFilename(image->filename,
+    fpx_status=FPX_OpenImageByFilename(image->filename,(char *) NULL,
 #endif
-      NULL,&width,&height,&tile_width,&tile_height,&colorspace,&flashpix);
+      &width,&height,&tile_width,&tile_height,&colorspace,&flashpix);
   }
   if (fpx_status == FPX_LOW_MEMORY_ERROR)
     {
       FPX_ClearSystem();
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenWarning,"Unable to open FPX file",image);
+      ThrowFileException(exception,FileOpenError,"UnableToOpenFile",
+        image->filename);
+      DestroyImageList(image);
+      return(False);
+    }
+  if (colorspace.numberOfComponents == 0)
+    {
+      FPX_ClearSystem();
+      ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
     }
   if (image_info->view == (char *) NULL)
     {
@@ -240,8 +277,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       aspect_ratio=(float) width/height;
       fpx_status=FPX_GetImageResultAspectRatio(flashpix,&aspect_ratio);
       if (fpx_status != FPX_OK)
-        ThrowReaderException(DelegateWarning,"Unable to read aspect ratio",
-          image);
+        ThrowReaderException(DelegateError,"UnableToReadAspectRatio");
       if (width != (unsigned long) ((aspect_ratio*height)+0.5))
         Swap(width,height);
     }
@@ -249,7 +285,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (fpx_status != FPX_OK)
     {
       FPX_ClearSystem();
-      ThrowReaderException(FileOpenWarning,"Unable to read summary info",image);
+      ThrowReaderException(DelegateError,"UnableToReadSummaryInfo");
     }
   if (summary_info.title_valid)
     if ((summary_info.title.length != 0) &&
@@ -261,18 +297,17 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image label.
         */
-        label=(char *) AcquireMemory(summary_info.title.length+1);
+        label=(char *) AcquireMagickMemory(summary_info.title.length+1);
         if (label == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         (void) strncpy(label,(char *) summary_info.title.ptr,
           summary_info.title.length);
         label[summary_info.title.length]='\0';
-        (void) SetImageAttribute(image,"Label",label);
-        LiberateMemory((void **) &label);
+        (void) SetImageAttribute(image,"label",label);
+        label=(char *) RelinquishMagickMemory(label);
       }
   if (summary_info.comments_valid)
     if ((summary_info.comments.length != 0) &&
@@ -284,40 +319,38 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Note image comment.
         */
-        comments=(char *) AcquireMemory(summary_info.comments.length+1);
+        comments=(char *) AcquireMagickMemory(summary_info.comments.length+1);
         if (comments == (char *) NULL)
           {
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
         (void) strncpy(comments,(char *) summary_info.comments.ptr,
           summary_info.comments.length);
         comments[summary_info.comments.length]='\0';
-        (void) SetImageAttribute(image,"Comment",comments);
-        LiberateMemory((void **) &comments);
+        (void) SetImageAttribute(image,"comment",comments);
+        comments=(char *) RelinquishMagickMemory(comments);
       }
   /*
-    Determine resolution by subimage specification.
+    Determine resolution by scene specification.
   */
   for (i=1; ; i++)
-    if (((width >> i) < tile_width) ||
-        ((height >> i) < tile_height))
+    if (((width >> i) < tile_width) || ((height >> i) < tile_height))
       break;
-  subimage=i;
-  if (image_info->subrange != 0)
-    while (subimage > image_info->subimage)
+  scene=i;
+  if (image_info->number_scenes != 0)
+    while (scene > image_info->scene)
     {
       width>>=1;
       height>>=1;
-      subimage--;
+      scene--;
     }
   if (image_info->size != (char *) NULL)
     while ((width > image->columns) || (height > image->rows))
     {
       width>>=1;
       height>>=1;
-      subimage--;
+      scene--;
     }
   image->depth=8;
   image->columns=width;
@@ -329,29 +362,28 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Create linear colormap.
       */
-      if (!AllocateImageColormap(image,MaxRGB+1))
+      if (AllocateImageColormap(image,MaxColormapSize) == False)
         {
           FPX_ClearSystem();
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
-  if (image_info->ping)
+  if (image_info->ping != False)
     {
-      CloseBlob(image);
-      return(image);
+      (void) FPX_CloseImage(flashpix);
+      FPX_ClearSystem();
+      return(GetFirstImageInList(image));
     }
   /*
     Allocate memory for the image and pixel buffer.
   */
-  scanline=(unsigned char *) AcquireMemory(colorspace.numberOfComponents*
+  scanline=(unsigned char *) AcquireMagickMemory(colorspace.numberOfComponents*
     image->columns*(tile_height+1));
   if (scanline == (unsigned char *) NULL)
     {
       FPX_ClearSystem();
       (void) FPX_CloseImage(flashpix);
-      ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
@@ -381,7 +413,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Initialize image pixels.
   */
-  for (y=0; y < (int) image->rows; y++)
+  for (y=0; y < (long) image->rows; y++)
   {
     q=SetImagePixels(image,0,y,image->columns,1);
     if (q == (PixelPacket *) NULL)
@@ -394,19 +426,18 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         if (image_info->view != (char *) NULL)
           fpx_status=FPX_ReadImageRectangle(flashpix,0,y,image->columns,y+
-            tile_height-1,subimage,&fpx_info);
+            tile_height-1,scene,&fpx_info);
         else
-          fpx_status=FPX_ReadImageTransformRectangle(flashpix,0.0F,(float) y/
-            image->rows,(float) image->columns/image->rows,(float) (y+
-            tile_height-1)/image->rows,(long) image->columns,(long) tile_height,
-            &fpx_info);
+          fpx_status=FPX_ReadImageTransformRectangle(flashpix,0.0F,
+            (float) y/image->rows,(float) image->columns/image->rows,
+            (float) (y+tile_height-1)/image->rows,(long) image->columns,
+            (long) tile_height,&fpx_info);
         if (fpx_status == FPX_LOW_MEMORY_ERROR)
           {
-            LiberateMemory((void **) &scanline);
+            scanline=(char *) RelinquishMagickMemory(scanline);
             (void) FPX_CloseImage(flashpix);
             FPX_ClearSystem();
-            ThrowReaderException(ResourceLimitWarning,
-              "Memory allocation failed",image);
+            ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
           }
       }
     /*
@@ -416,51 +447,40 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     g=green_component->theData+(y % tile_height)*green_component->lineStride;
     b=blue_component->theData+(y % tile_height)*blue_component->lineStride;
     a=alpha_component->theData+(y % tile_height)*alpha_component->lineStride;
-    for (x=0; x < (int) image->columns; x++)
+    for (x=0; x < (long) image->columns; x++)
     {
       if (fpx_info.numberOfComponents > 2)
         {
-          q->red=UpScale(*r);
-          q->green=UpScale(*g);
-          q->blue=UpScale(*b);
+          q->red=ScaleCharToQuantum(*r);
+          q->green=ScaleCharToQuantum(*g);
+          q->blue=ScaleCharToQuantum(*b);
         }
       else
         {
-          index=UpScale(*r);
+          index=ScaleCharToQuantum(*r);
           indexes[x]=index;
           q->red=index;
           q->green=index;
           q->blue=index;
         }
-      if (image->matte)
-        q->opacity=MaxRGB-UpScale(*a);
+      if (image->matte != False)
+        q->opacity=ScaleCharToQuantum(255-*a);
       q++;
       r+=red_component->columnStride;
       g+=green_component->columnStride;
       b+=blue_component->columnStride;
       a+=alpha_component->columnStride;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == False)
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+      if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
+        break;
   }
-  LiberateMemory((void **) &scanline);
+  scanline=(char *) RelinquishMagickMemory(scanline);
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  if (image->temporary)
-    {
-      (void) remove(image->filename);
-      image->temporary=False;
-    }
-  return(image);
-}
-#else
-static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
-{
-  ThrowException(exception,MissingDelegateWarning,
-    "FPX library is not available",image_info->filename);
-  return((Image *) NULL);
+  return(GetFirstImageInList(image));
 }
 #endif
 
@@ -475,7 +495,7 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFPXImage adds attributes for the FPX image format to
+%  RegisterFPXImage() adds attributes for the FPX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -493,13 +513,16 @@ ModuleExport void RegisterFPXImage(void)
     *entry;
 
   entry=SetMagickInfo("FPX");
-  entry->decoder=ReadFPXImage;
-  entry->encoder=WriteFPXImage;
+#if defined(HasFPX)
+  entry->decoder=(DecoderHandler *) ReadFPXImage;
+  entry->encoder=(EncoderHandler *) WriteFPXImage;
+#endif
   entry->adjoin=False;
-  entry->blob_support=False;
-  entry->description=AllocateString("FlashPix Format");
-  entry->module=AllocateString("FPX");
-  RegisterMagickInfo(entry);
+  entry->seekable_stream=True;
+  entry->magick=(MagickHandler *) IsFPX;
+  entry->description=AcquireString("FlashPix Format");
+  entry->module=AcquireString("FPX");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -513,7 +536,7 @@ ModuleExport void RegisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFPXImage removes format registrations made by the
+%  UnregisterFPXImage() removes format registrations made by the
 %  FPX module from the list of supported formats.
 %
 %  The format of the UnregisterFPXImage method is:
@@ -523,7 +546,7 @@ ModuleExport void RegisterFPXImage(void)
 */
 ModuleExport void UnregisterFPXImage(void)
 {
-  UnregisterMagickInfo("FPX");
+  (void) UnregisterMagickInfo("FPX");
 }
 
 #if defined(HasFPX)
@@ -538,7 +561,7 @@ ModuleExport void UnregisterFPXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteFPXImage writes an image in the FlashPix image format.  This
+%  WriteFPXImage() writes an image in the FlashPix image format.  This
 %  method was contributed by BillR@corbis.com.
 %
 %  The format of the WriteFPXImage method is:
@@ -551,9 +574,9 @@ ModuleExport void UnregisterFPXImage(void)
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
@@ -565,45 +588,45 @@ static void ColorTwistMultiply(FPXColorTwistMatrix first,
     Matrix multiply.
   */
   assert(color_twist != (FPXColorTwistMatrix *) NULL);
-  color_twist->byy=(first.byy*second.byy)+(first.byc1*second.bc1y)+(first.byc2*
-    second.bc2y)+(first.dummy1_zero*second.dummy4_zero);
+  color_twist->byy=(first.byy*second.byy)+(first.byc1*second.bc1y)+
+    (first.byc2*second.bc2y)+(first.dummy1_zero*second.dummy4_zero);
   color_twist->byc1=(first.byy*second.byc1)+(first.byc1*second.bc1c1)+
     (first.byc2*second.bc2c1)+(first.dummy1_zero*second.dummy5_zero);
   color_twist->byc2=(first.byy*second.byc2)+(first.byc1*second.bc1c2)+
     (first.byc2*second.bc2c2)+(first.dummy1_zero*second.dummy6_zero);
-  color_twist->dummy1_zero=(first.byy*second.dummy1_zero)+(first.byc1*
-    second.dummy2_zero)+(first.byc2*second.dummy3_zero)+(first.dummy1_zero*
-    second.dummy7_one);
+  color_twist->dummy1_zero=(first.byy*second.dummy1_zero)+
+    (first.byc1*second.dummy2_zero)+(first.byc2*second.dummy3_zero)+
+    (first.dummy1_zero*second.dummy7_one);
   color_twist->bc1y=(first.bc1y*second.byy)+(first.bc1c1*second.bc1y)+
     (first.bc1c2*second.bc2y)+(first.dummy2_zero*second.dummy4_zero);
   color_twist->bc1c1=(first.bc1y*second.byc1)+(first.bc1c1*second.bc1c1)+
     (first.bc1c2*second.bc2c1)+(first.dummy2_zero*second.dummy5_zero);
   color_twist->bc1c2=(first.bc1y*second.byc2)+(first.bc1c1*second.bc1c2)+
     (first.bc1c2*second.bc2c2)+(first.dummy2_zero*second.dummy6_zero);
-  color_twist->dummy2_zero=(first.bc1y*second.dummy1_zero)+(first.bc1c1*
-    second.dummy2_zero)+(first.bc1c2*second.dummy3_zero)+(first.dummy2_zero*
-    second.dummy7_one);
+  color_twist->dummy2_zero=(first.bc1y*second.dummy1_zero)+
+    (first.bc1c1*second.dummy2_zero)+(first.bc1c2*second.dummy3_zero)+
+    (first.dummy2_zero*second.dummy7_one);
   color_twist->bc2y=(first.bc2y*second.byy)+(first.bc2c1*second.bc1y)+
     (first.bc2c2*second.bc2y)+(first.dummy3_zero*second.dummy4_zero);
   color_twist->bc2c1=(first.bc2y*second.byc1)+(first.bc2c1*second.bc1c1)+
     (first.bc2c2*second.bc2c1)+(first.dummy3_zero*second.dummy5_zero);
   color_twist->bc2c2=(first.bc2y*second.byc2)+(first.bc2c1*second.bc1c2)+
     (first.bc2c2*second.bc2c2)+(first.dummy3_zero*second.dummy6_zero);
-  color_twist->dummy3_zero=(first.bc2y*second.dummy1_zero)+(first.bc2c1*
-    second.dummy2_zero)+(first.bc2c2*second.dummy3_zero)+(first.dummy3_zero*
-    second.dummy7_one);
-  color_twist->dummy4_zero=(first.dummy4_zero*second.byy)+(first.dummy5_zero*
-    second.bc1y)+(first.dummy6_zero*second.bc2y)+(first.dummy7_one*
-    second.dummy4_zero);
-  color_twist->dummy5_zero=(first.dummy4_zero*second.byc1)+(first.dummy5_zero*
-    second.bc1c1)+(first.dummy6_zero*second.bc2c1)+(first.dummy7_one*
-    second.dummy5_zero);
-  color_twist->dummy6_zero=(first.dummy4_zero*second.byc2)+(first.dummy5_zero*
-    second.bc1c2)+(first.dummy6_zero*second.bc2c2)+(first.dummy7_one*
-    second.dummy6_zero);
+  color_twist->dummy3_zero=(first.bc2y*second.dummy1_zero)+
+    (first.bc2c1*second.dummy2_zero)+(first.bc2c2*second.dummy3_zero)+
+    (first.dummy3_zero*second.dummy7_one);
+  color_twist->dummy4_zero=(first.dummy4_zero*second.byy)+
+    (first.dummy5_zero*second.bc1y)+(first.dummy6_zero*second.bc2y)+
+    (first.dummy7_one*second.dummy4_zero);
+  color_twist->dummy5_zero=(first.dummy4_zero*second.byc1)+
+    (first.dummy5_zero*second.bc1c1)+(first.dummy6_zero*second.bc2c1)+
+    (first.dummy7_one*second.dummy5_zero);
+  color_twist->dummy6_zero=(first.dummy4_zero*second.byc2)+
+    (first.dummy5_zero*second.bc1c2)+(first.dummy6_zero*second.bc2c2)+
+    (first.dummy7_one*second.dummy6_zero);
   color_twist->dummy7_one=(first.dummy4_zero*second.dummy1_zero)+
-    (first.dummy5_zero*second.dummy2_zero)+(first.dummy6_zero*
-    second.dummy3_zero)+(first.dummy7_one*second.dummy7_one);
+    (first.dummy5_zero*second.dummy2_zero)+
+    (first.dummy6_zero*second.dummy3_zero)+(first.dummy7_one*second.dummy7_one);
 }
 
 static void SetBrightness(double brightness,FPXColorTwistMatrix *color_twist)
@@ -748,12 +771,18 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     colorspace =
     {
       TRUE, 4,
-      NIFRGB_R, DATA_TYPE_UNSIGNED_BYTE,
-      NIFRGB_G, DATA_TYPE_UNSIGNED_BYTE,
-      NIFRGB_B, DATA_TYPE_UNSIGNED_BYTE,
-      ALPHA, DATA_TYPE_UNSIGNED_BYTE
+      {
+        { NIFRGB_R, DATA_TYPE_UNSIGNED_BYTE },
+        { NIFRGB_G, DATA_TYPE_UNSIGNED_BYTE },
+        { NIFRGB_B, DATA_TYPE_UNSIGNED_BYTE },
+        { ALPHA, DATA_TYPE_UNSIGNED_BYTE }
+      }
     };
 
+  const ImageAttribute
+    *comment,
+    *label;
+
   FPXCompressionOption
     compression;
 
@@ -769,13 +798,10 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   FPXSummaryInformation
     summary_info;
 
-  Image
-    fpx_image;
-
-  int
+  long
     y;
 
-  register int
+  register long
     i;
 
   unsigned char
@@ -789,80 +815,70 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     tile_height,
     tile_width;
 
-  ImageAttribute
-    *comment,
-    *label;
-
   /*
     Open input file.
   */
-  status=OpenBlob(image_info,image,WriteBinaryType);
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  if ((image->file != stdout) && !image->pipe)
-    (void) remove(image->filename);
-  else
-    {
-      /*
-        Write standard output or pipe to temporary file.
-      */
-      fpx_image=(*image);
-      TemporaryFilename(image->filename);
-      image->temporary=True;
-    }
+    return(status);
   CloseBlob(image);
-  image->depth=8;
-  TransformRGBImage(image,RGBColorspace);
   /*
     Initialize FPX toolkit.
   */
+  image->depth=8;
+  (void) SetImageColorspace(image,RGBColorspace);
   memory_limit=20000000;
   fpx_status=FPX_SetToolkitMemoryLimit(&memory_limit);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(ResourceLimitWarning,
-      "Unable to initialize FPX library",image);
+    ThrowWriterException(DelegateError,"UnableToInitializeFPXLibrary");
   tile_width=64;
   tile_height=64;
   colorspace.numberOfComponents=3;
-  if (image->matte)
+  if (image->matte != False)
     colorspace.numberOfComponents=4;
-  if (image->storage_class != DirectClass)
-    if (IsGrayImage(image))
-      {
-        colorspace.numberOfComponents=1;
-        colorspace.theComponents[0].myColor=MONOCHROME;
-      }
+  if ((image_info->type != TrueColorType) &&
+      IsGrayImage(image,&image->exception))
+    {
+      colorspace.numberOfComponents=1;
+      colorspace.theComponents[0].myColor=MONOCHROME;
+    }
   background_color.color1_value=0;
   background_color.color2_value=0;
   background_color.color3_value=0;
   background_color.color4_value=0;
   compression=NONE;
-  if (image_info->compression == JPEGCompression)
+  if (image->compression == JPEGCompression)
     compression=JPEG_UNSPECIFIED;
   {
 #if defined(macintosh)
     FSSpec
       fsspec;
 
-    FilenameToFSSpec(image->filename,&fsspec);
-    fpx_status=FPX_CreateImageByFilename((const FSSpec &) fsspec,
+    FilenameToFSSpec(filename,&fsspec);
+    fpx_status=FPX_CreateImageByFilename((const FSSpec &) fsspec,image->columns,
 #else
-    fpx_status=FPX_CreateImageByFilename(image->filename,
+    fpx_status=FPX_CreateImageByFilename(image->filename,image->columns,
 #endif
-      image->columns,image->rows,tile_width,tile_height,colorspace,
-      background_color,compression,&flashpix);
+      image->rows,tile_width,tile_height,colorspace,background_color,
+      compression,&flashpix);
   }
   if (fpx_status != FPX_OK)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-  if (image_info->compression == JPEGCompression)
+    return(status);
+  if (compression == JPEG_UNSPECIFIED)
     {
       /*
         Initialize the compression by quality for the entire image.
       */
-      fpx_status=
-        FPX_SetJPEGCompression(flashpix,(unsigned short) (image_info->quality));
+      fpx_status=FPX_SetJPEGCompression(flashpix,(unsigned short)
+        image_info->quality == 0 ? 75 : image_info->quality);
       if (fpx_status != FPX_OK)
-        ThrowWriterException(DelegateWarning,"Unable to set JPEG level",image);
+        ThrowWriterException(DelegateError,"UnableToSetJPEGLevel");
     }
   /*
     Set image summary info.
@@ -885,58 +901,54 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   summary_info.thumbnail_valid=False;
   summary_info.appname_valid=False;
   summary_info.security_valid=False;
-  label=GetImageAttribute(image,"Label");
+  label=GetImageAttribute(image,"label");
   if (label != (ImageAttribute *) NULL)
     {
       /*
         Note image label.
       */
       summary_info.title_valid=True;
-      summary_info.title.length=Extent(label->value);
+      summary_info.title.length=strlen(label->value);
       summary_info.title.ptr=(unsigned char *)
-        AcquireMemory(Extent(label->value)+1);
-      if (summary_info.title.ptr != (unsigned char *) NULL)
-        (void) strcpy((char *) summary_info.title.ptr,label->value);
-      else
-        ThrowWriterException(DelegateWarning,"Unable to set image title",
-          image);
+        AcquireMagickMemory(strlen(label->value)+1);
+      if (summary_info.title.ptr == (unsigned char *) NULL)
+        ThrowWriterException(DelegateError,"UnableToSetImageTitle");
+      (void) CopyMagickString((char *) summary_info.title.ptr,label->value,
+        MaxTextExtent);
     }
-  comment=GetImageAttribute(image,"Comment");
+  comment=GetImageAttribute(image,"comment");
   if (comment != (ImageAttribute *) NULL)
     {
       /*
         Note image comment.
       */
       summary_info.comments_valid=True;
-      summary_info.comments.length=Extent(comment->value);
+      summary_info.comments.length=strlen(comment->value);
       summary_info.comments.ptr=(unsigned char *)
-        AcquireMemory(Extent(comment->value)+1);
-      if (summary_info.comments.ptr != (unsigned char *) NULL)
-        (void) strcpy((char *) summary_info.comments.ptr,comment->value);
-      else
-        ThrowWriterException(DelegateWarning,"Unable to set image comments",
-          image);
+        AcquireMagickMemory(strlen(comment->value)+1);
+      if (summary_info.comments.ptr == (unsigned char *) NULL)
+        ThrowWriterException(DelegateError,"UnableToSetImageComment");
+      (void) strcpy((char *) summary_info.comments.ptr,comment->value);
     }
   fpx_status=FPX_SetSummaryInformation(flashpix,&summary_info);
   if (fpx_status != FPX_OK)
-    ThrowWriterException(DelegateWarning,"Unable to set summary info",image);
+    ThrowWriterException(DelegateError,"UnableToSetSummaryInfo");
   /*
     Allocate pixels.
   */
   pixels=(unsigned char *)
-    AcquireMemory(colorspace.numberOfComponents*image->columns);
+    AcquireMagickMemory(colorspace.numberOfComponents*image->columns);
   if (pixels == (unsigned char *) NULL)
     {
       (void) FPX_CloseImage(flashpix);
       FPX_ClearSystem();
-      ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-        image);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
   /*
     Initialize FlashPix image description.
   */
   fpx_info.numberOfComponents=colorspace.numberOfComponents;
-  for (i=0; i < fpx_info.numberOfComponents; i++)
+  for (i=0; i < (long) fpx_info.numberOfComponents; i++)
   {
     fpx_info.components[i].myColorType.myDataType=DATA_TYPE_UNSIGNED_BYTE;
     fpx_info.components[i].horzSubSampFactor=1;
@@ -954,14 +966,14 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
   /*
     Write image scanlines.
   */
-  for (y=0; y < (int) image->rows; y++)
+  for (y=0; y < (long) image->rows; y++)
   {
-    if (!GetImagePixels(image,0,y,image->columns,1))
+    if (AcquireImagePixels(image,0,y,image->columns,1,&image->exception) == False)
       break;
     if (fpx_info.numberOfComponents == 1)
       (void) PopImagePixels(image,GrayQuantum,pixels);
     else
-      if (!image->matte)
+      if (image->matte == False)
         (void) PopImagePixels(image,RGBQuantum,pixels);
       else
         (void) PopImagePixels(image,RGBAQuantum,pixels);
@@ -969,7 +981,8 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
     if (fpx_status != FPX_OK)
       break;
     if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+      if (MagickMonitor(SaveImageTag,y,image->rows,&image->exception) == False)
+        break;
   }
   if (image_info->view != (char *) NULL)
     {
@@ -1050,7 +1063,7 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
       if (0)
         {
           /*
-            Color color_twist.
+            Color color twist.
           */
           SetBrightness(0.5,&color_twist);
           SetSaturation(0.5,&color_twist);
@@ -1061,75 +1074,42 @@ static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
         {
           fpx_status=FPX_SetImageAffineMatrix(flashpix,&affine);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set affine matrix",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAffineMatrix");
         }
       if (aspect_ratio_valid)
         {
           fpx_status=FPX_SetImageResultAspectRatio(flashpix,&aspect_ratio);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set aspect ratio",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetAspectRatio");
         }
       if (color_twist_valid)
         {
           fpx_status=FPX_SetImageColorTwistMatrix(flashpix,&color_twist);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set color color twist",image);
+            ThrowWriterException(DelegateError,"UnableToSetColorTwist");
         }
       if (contrast_valid)
         {
           fpx_status=FPX_SetImageContrastAdjustment(flashpix,&contrast);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,"Unable to set contrast",
-              image);
+            ThrowWriterException(DelegateError,"UnableToSetContrast");
         }
       if (sharpen_valid)
         {
           fpx_status=FPX_SetImageFilteringValue(flashpix,&sharpen);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set filtering value",image);
+            ThrowWriterException(DelegateError,"UnableToSetFilteringValue");
         }
       if (view_rect_valid)
         {
           fpx_status=FPX_SetImageROI(flashpix,&view_rect);
           if (fpx_status != FPX_OK)
-            ThrowWriterException(DelegateWarning,
-              "Unable to set region of interest",image);
+            ThrowWriterException(DelegateError,"UnableToSetRegionOfInterest");
         }
     }
   (void) FPX_CloseImage(flashpix);
   FPX_ClearSystem();
-  LiberateMemory((void **) &pixels);
-  if (image->temporary)
-    {
-      FILE
-        *file;
-
-      int
-        c;
-
-      /*
-        Copy temporary file to standard output or pipe.
-      */
-      file=fopen(image->filename,ReadBinaryType);
-      if (file == (FILE *) NULL)
-        ThrowWriterException(FileOpenWarning,"Unable to open file",image);
-      for (c=fgetc(file); c != EOF; c=fgetc(file))
-        (void) fputc(c,fpx_image.file);
-      (void) fclose(file);
-      (void) remove(image->filename);
-      image->temporary=False;
-      CloseBlob(&fpx_image);
-    }
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   return(True);
 }
-#else
-static unsigned int WriteFPXImage(const ImageInfo *image_info,Image *image)
-{
-  ThrowBinaryException(MissingDelegateWarning,"FPX library is not available",
-    image->filename);
-}
 #endif
