@@ -10,39 +10,26 @@
 %                            R  R    GGG   BBBB                               %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                     Read/Write Raw RGB Image Format.                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2001 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,9 +39,20 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "defines.h"
-
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/constitute.h"
+#include "magick/error.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
+#include "magick/utility.h"
+
 /*
   Forward declarations.
 */
@@ -72,7 +70,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadRGBImage reads an image of raw red, green, and blue bytes and
+%  ReadRGBImage() reads an image of raw red, green, and blue samples and
 %  returns it.  It allocates the memory necessary for the new Image structure
 %  and returns a pointer to the new image.
 %
@@ -86,7 +84,7 @@ static unsigned int
 %      reading.  A null image is returned if there is a memory shortage or
 %      if the image cannot be read.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -97,64 +95,86 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  int
-    count,
+  long
     y;
 
-  register int
+  register long
     i,
     x;
 
+  register PixelPacket
+    *q;
+
+  ssize_t
+    count;
+
+  size_t
+    packet_size;
+
   unsigned char
     *scanline;
 
   unsigned int
-    packet_size,
     status;
 
+  unsigned long
+    width;
+
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionWarning,"Must specify image size",image);
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open image file.
       */
-      status=OpenBlob(image_info,image,ReadBinaryType);
+      status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == False)
-        ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+        {
+          DestroyImageList(image);
+          return((Image *) NULL);
+        }
       for (i=0; i < image->offset; i++)
         (void) ReadBlobByte(image);
     }
   /*
     Allocate memory for a scanline.
   */
-  packet_size=image->depth > 8 ? 6 : 3;
+  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
     {
       image->matte=True;
-      packet_size=image->depth > 8 ? 8 : 4;
+      packet_size=(size_t) (image->depth > 8 ? 8 : 4);
     }
   scanline=(unsigned char *)
-    AcquireMemory(packet_size*image->tile_info.width);
+    AcquireMagickMemory(packet_size*image->extract_info.width);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",image);
-  if (image_info->subrange != 0)
-    while (image->scene < image_info->subimage)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  if (image_info->number_scenes != 0)
+    while (image->scene < image_info->scene)
     {
       /*
         Skip to next image.
       */
       image->scene++;
-      for (y=0; y < (int) image->rows; y++)
-        (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+      for (y=0; y < (long) image->rows; y++)
+        (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
     }
-  x=packet_size*image->tile_info.x;
+  x=(long) (packet_size*image->extract_info.x);
   do
   {
     /*
       Convert raster image to pixel packets.
     */
+    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+        break;
     switch (image_info->interlace)
     {
       case NoInterlace:
@@ -163,27 +183,30 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           No interlacing:  RGBRGBRGBRGBRGBRGB...
         */
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < image->extract_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          if (!SetImagePixels(image,0,y,image->columns,1))
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
+          q=SetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
             break;
-          if (!image->matte)
+          if (image->matte == False)
             (void) PushImagePixels(image,RGBQuantum,scanline+x);
           else
             (void) PushImagePixels(image,RGBAQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == False)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
+                break;
         }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
       case LineInterlace:
@@ -191,41 +214,45 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           Line interlacing:  RRR...GGG...BBB...RRR...GGG...BBB...
         */
-        packet_size=image->depth > 8 ? 2 : 1;
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-        for (y=0; y < (int) image->rows; y++)
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+        for (y=0; y < image->extract_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,
+            scanline);
+        for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          if (!SetImagePixels(image,0,y,image->columns,1))
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
+          q=SetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,RedQuantum,scanline+x);
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
           (void) PushImagePixels(image,GreenQuantum,scanline+x);
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
           (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (image->matte)
+          if (image->matte != False)
             {
-              (void) ReadBlob(image,packet_size*image->tile_info.width,
+              (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              (void) PushImagePixels(image,OpacityQuantum,scanline+x);
+              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
             }
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == False)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(LoadImageText,y,image->rows);
+              if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
+                break;
         }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         break;
       }
       case PlaneInterlace:
       case PartitionInterlace:
       {
-        unsigned int
+        unsigned long
           span;
 
         /*
@@ -234,85 +261,101 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("R",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
-        packet_size=image->depth > 8 ? 2 : 1;
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+        for (y=0; y < image->extract_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         i=0;
-        span=image->rows*(image->matte ? 4 : 3);
-        for (y=0; y < (int) image->rows; y++)
+        span=image->rows*(image->matte != False ? 4 : 3);
+        for (y=0; y < (long) image->rows; y++)
         {
           if ((y > 0) || (image->previous == (Image *) NULL))
-            (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          if (!SetImagePixels(image,0,y,image->columns,1))
+            (void) ReadBlob(image,packet_size*image->extract_info.width,
+              scanline);
+          q=SetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,RedQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == False)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
+                break;
           i++;
         }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < image->extract_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        for (y=0; y < (long) image->rows; y++)
         {
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          q=GetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,GreenQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == False)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
+                break;
           i++;
         }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
-            status=OpenBlob(image_info,image,ReadBinaryType);
+            status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
             if (status == False)
-              ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+              {
+                DestroyImageList(image);
+                return((Image *) NULL);
+              }
           }
-        for (y=0; y < image->tile_info.y; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < image->extract_info.y; y++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        for (y=0; y < (long) image->rows; y++)
         {
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+          q=GetImagePixels(image,0,y,image->columns,1);
+          if (q == (PixelPacket *) NULL)
             break;
           (void) PushImagePixels(image,BlueQuantum,scanline+x);
-          if (!SyncImagePixels(image))
+          if (SyncImagePixels(image) == False)
             break;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(i,span))
-              MagickMonitor(LoadImageText,i,span);
+              if (MagickMonitor(LoadImageTag,i,span,&image->exception) == False)
+                break;
           i++;
         }
-        count=image->tile_info.height-image->rows-image->tile_info.y;
-        for (y=0; y < count; y++)
-          (void) ReadBlob(image,packet_size*image->tile_info.width,scanline);
-        if (image->matte)
+        width=image->extract_info.height-image->rows-image->extract_info.y;
+        for (i=0; i < (long) width; i++)
+          (void) ReadBlob(image,packet_size*image->extract_info.width,scanline);
+        if (image->matte != False)
           {
             /*
               Read matte channel.
@@ -321,46 +364,62 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,ReadBinaryType);
+                status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
                 if (status == False)
-                  ThrowReaderException(FileOpenWarning,"Unable to open file",
-                    image);
+                  {
+                    DestroyImageList(image);
+                    return((Image *) NULL);
+                  }
               }
-            for (y=0; y < image->tile_info.y; y++)
-              (void) ReadBlob(image,packet_size*image->tile_info.width,
+            for (y=0; y < image->extract_info.y; y++)
+              (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-            for (y=0; y < (int) image->rows; y++)
+            for (y=0; y < (long) image->rows; y++)
             {
-              (void) ReadBlob(image,packet_size*image->tile_info.width,
+              (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
-              if (!GetImagePixels(image,0,y,image->columns,1))
+              q=GetImagePixels(image,0,y,image->columns,1);
+              if (q == (PixelPacket *) NULL)
                 break;
-              (void) PushImagePixels(image,OpacityQuantum,scanline+x);
-              if (!SyncImagePixels(image))
+              (void) PushImagePixels(image,AlphaQuantum,scanline+x);
+              if (SyncImagePixels(image) == False)
                 break;
               if (image->previous == (Image *) NULL)
                 if (QuantumTick(i,span))
-                  MagickMonitor(LoadImageText,i,span);
+                  {
+                    status=MagickMonitor(LoadImageTag,i,span,&image->exception);
+                    if (status == False)
+                      break;
+                  }
               i++;
             }
-            count=image->tile_info.height-image->rows-image->tile_info.y;
-            for (y=0; y < count; y++)
-              (void) ReadBlob(image,packet_size*image->tile_info.width,
+            width=image->extract_info.height-image->rows-image->extract_info.y;
+            for (i=0; i < (long) width; i++)
+              (void) ReadBlob(image,packet_size*image->extract_info.width,
                 scanline);
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strcpy(image->filename,image_info->filename);
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
         break;
       }
     }
+    if (EOFBlob(image) != False)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    count=ReadBlob(image,packet_size*image->tile_info.width,scanline);
-    if (count > 0)
+    if (image_info->interlace == PartitionInterlace)
+      break;
+    count=ReadBlob(image,packet_size*image->extract_info.width,scanline);
+    if (count != 0)
       {
         /*
           Allocate next image structure.
@@ -368,18 +427,19 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImages(image);
+            DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),image->filesize);
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
-  } while (count > 0);
-  LiberateMemory((void **) &scanline);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
+  } while (count != 0);
+  scanline=(unsigned char *) RelinquishMagickMemory(scanline);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -393,7 +453,7 @@ static Image *ReadRGBImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterRGBImage adds attributes for the RGB image format to
+%  RegisterRGBImage() adds attributes for the RGB image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -411,19 +471,19 @@ ModuleExport void RegisterRGBImage(void)
     *entry;
 
   entry=SetMagickInfo("RGB");
-  entry->decoder=ReadRGBImage;
-  entry->encoder=WriteRGBImage;
+  entry->decoder=(DecoderHandler *) ReadRGBImage;
+  entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=True;
-  entry->description=AllocateString("Raw red, green, and blue bytes");
-  entry->module=AllocateString("RGB");
-  RegisterMagickInfo(entry);
+  entry->description=AcquireString("Raw red, green, and blue samples");
+  entry->module=AcquireString("RGB");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RGBA");
-  entry->decoder=ReadRGBImage;
-  entry->encoder=WriteRGBImage;
+  entry->decoder=(DecoderHandler *) ReadRGBImage;
+  entry->encoder=(EncoderHandler *) WriteRGBImage;
   entry->raw=True;
-  entry->description=AllocateString("Raw red, green, blue, and matte bytes");
-  entry->module=AllocateString("RGB");
-  RegisterMagickInfo(entry);
+  entry->description=AcquireString("Raw red, green, blue, and matte samples");
+  entry->module=AcquireString("RGB");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -437,7 +497,7 @@ ModuleExport void RegisterRGBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterRGBImage removes format registrations made by the
+%  UnregisterRGBImage() removes format registrations made by the
 %  RGB module from the list of supported formats.
 %
 %  The format of the UnregisterRGBImage method is:
@@ -447,8 +507,8 @@ ModuleExport void RegisterRGBImage(void)
 */
 ModuleExport void UnregisterRGBImage(void)
 {
-  UnregisterMagickInfo("RGB");
-  UnregisterMagickInfo("RGBA");
+  (void) UnregisterMagickInfo("RGB");
+  (void) UnregisterMagickInfo("RGBA");
 }
 
 /*
@@ -462,7 +522,7 @@ ModuleExport void UnregisterRGBImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteRGBImage writes an image to a file in red, green, and blue
+%  WriteRGBImage() writes an image to a file in red, green, and blue
 %  rasterfile format.
 %
 %  The format of the WriteRGBImage method is:
@@ -475,42 +535,55 @@ ModuleExport void UnregisterRGBImage(void)
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
 static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
 {
-  int
+  long
     y;
 
+  MagickOffsetType
+    scene;
+
+  register const PixelPacket
+    *p;
+
+  size_t
+    packet_size;
+
   unsigned char
     *pixels;
 
   unsigned int
-    packet_size,
-    scene,
     status;
 
   /*
     Allocate memory for pixels.
   */
-  packet_size=image->depth > 8 ? 6 : 3;
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  packet_size=(size_t) (image->depth > 8 ? 6 : 3);
   if (LocaleCompare(image_info->magick,"RGBA") == 0)
-    packet_size=image->depth > 8 ? 8 : 4;
-  pixels=(unsigned char *) AcquireMemory(packet_size*image->columns);
+    packet_size=(size_t) (image->depth > 8 ? 8 : 4);
+  pixels=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (pixels == (unsigned char *) NULL)
-    ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",image);
+    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
   if (image_info->interlace != PartitionInterlace)
     {
       /*
         Open output image file.
       */
-      status=OpenBlob(image_info,image,WriteBinaryType);
+      status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
       if (status == False)
-        ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+        return(status);
     }
   scene=0;
   do
@@ -518,9 +591,9 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
     /*
       Convert MIFF to RGB raster pixels.
     */
-    TransformRGBImage(image,RGBColorspace);
+    (void) SetImageColorspace(image,RGBColorspace);
     if (LocaleCompare(image_info->magick,"RGBA") == 0)
-      if (!image->matte)
+      if (image->matte == False)
         SetImageOpacity(image,OpaqueOpacity);
     switch (image_info->interlace)
     {
@@ -530,9 +603,10 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         /*
           No interlacing:  RGBRGBRGBRGBRGBRGB...
         */
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < (long) image->rows; y++)
         {
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           if (LocaleCompare(image_info->magick,"RGBA") != 0)
             {
@@ -546,7 +620,12 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
             }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              MagickMonitor(SaveImageText,y,image->rows);
+              {
+                status=MagickMonitor(SaveImageTag,y,image->rows,
+                  &image->exception);
+                if (status == False)
+                  break;
+              }
         }
         break;
       }
@@ -555,23 +634,29 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         /*
           Line interlacing:  RRR...GGG...BBB...RRR...GGG...BBB...
         */
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < (long) image->rows; y++)
         {
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,RedQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,GreenQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           (void) PopImagePixels(image,BlueQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
           if (LocaleCompare(image_info->magick,"RGBA") == 0)
             {
-              (void) PopImagePixels(image,OpacityQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           if (QuantumTick(y,image->rows))
-            MagickMonitor(SaveImageText,y,image->rows);
+            {
+              status=MagickMonitor(SaveImageTag,y,image->rows,
+                &image->exception);
+              if (status == False)
+                break;
+            }
         }
         break;
       }
@@ -584,84 +669,97 @@ static unsigned int WriteRGBImage(const ImageInfo *image_info,Image *image)
         if (image_info->interlace == PartitionInterlace)
           {
             AppendImageFormat("R",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+              return(status);
           }
-        for (y=0; y < (int) image->rows; y++)
+        for (y=0; y < (long) image->rows; y++)
         {
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,RedQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("G",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+              return(status);
           }
-        MagickMonitor(SaveImageText,100,400);
-        for (y=0; y < (int) image->rows; y++)
+        if (MagickMonitor(SaveImageTag,100,400,&image->exception) == False)
+          break;
+        for (y=0; y < (long) image->rows; y++)
         {
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,GreenQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (image_info->interlace == PartitionInterlace)
           {
             CloseBlob(image);
             AppendImageFormat("B",image->filename);
-            status=OpenBlob(image_info,image,WriteBinaryType);
+            status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+              &image->exception);
             if (status == False)
-              ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+              return(status);
           }
-        MagickMonitor(SaveImageText,200,400);
-        for (y=0; y < (int) image->rows; y++)
+        if (MagickMonitor(SaveImageTag,200,400,&image->exception) == False)
+          break;
+        for (y=0; y < (long) image->rows; y++)
         {
-          if (!GetImagePixels(image,0,y,image->columns,1))
+          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          if (p == (const PixelPacket *) NULL)
             break;
           (void) PopImagePixels(image,BlueQuantum,pixels);
-          (void) WriteBlob(image,image->columns,pixels);
+          (void) WriteBlob(image,(size_t) image->columns,pixels);
         }
         if (LocaleCompare(image_info->magick,"RGBA") == 0)
           {
-            MagickMonitor(SaveImageText,300,400);
+            if (MagickMonitor(SaveImageTag,300,400,&image->exception) == False)
+              break;
             if (image_info->interlace == PartitionInterlace)
               {
                 CloseBlob(image);
                 AppendImageFormat("A",image->filename);
-                status=OpenBlob(image_info,image,WriteBinaryType);
+                status=OpenBlob(image_info,image,WriteBinaryBlobMode,
+                  &image->exception);
                 if (status == False)
-                  ThrowWriterException(FileOpenWarning,"Unable to open file",
-                    image);
+                  return(status);
               }
-            for (y=0; y < (int) image->rows; y++)
+            for (y=0; y < (long) image->rows; y++)
             {
-              if (!GetImagePixels(image,0,y,image->columns,1))
+              p=AcquireImagePixels(image,0,y,image->columns,1,
+                &image->exception);
+              if (p == (const PixelPacket *) NULL)
                 break;
-              (void) PopImagePixels(image,OpacityQuantum,pixels);
-              (void) WriteBlob(image,image->columns,pixels);
+              (void) PopImagePixels(image,AlphaQuantum,pixels);
+              (void) WriteBlob(image,(size_t) image->columns,pixels);
             }
           }
         if (image_info->interlace == PartitionInterlace)
-          (void) strcpy(image->filename,image_info->filename);
-        MagickMonitor(SaveImageText,400,400);
+          (void) CopyMagickString(image->filename,image_info->filename,
+            MaxTextExtent);
+        if (MagickMonitor(SaveImageTag,400,400,&image->exception) == False)
+          break;
         break;
       }
     }
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetNumberScenes(image));
-  } while (image_info->adjoin);
-  LiberateMemory((void **) &pixels);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
+  } while (image_info->adjoin != False);
+  pixels=(unsigned char *) RelinquishMagickMemory(pixels);
   CloseBlob(image);
   return(True);
 }
