@@ -10,39 +10,26 @@
 %                              T     GGG   A   A                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                    Read/Write Truevision Targa Image Format.                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2001 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,8 +39,19 @@
 /*
   Include declarations.
 */
-#include "magick.h"
-#include "defines.h"
+#include "magick/studio.h"
+#include "magick/attribute.h"
+#include "magick/blob.h"
+#include "magick/color_private.h"
+#include "magick/colorspace.h"
+#include "magick/error.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
@@ -72,7 +70,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadTGAImage reads a Truevision TGA image file and returns it.
+%  ReadTGAImage() reads a Truevision TGA image file and returns it.
 %  It allocates the memory necessary for the new Image structure and returns
 %  a pointer to the new image.
 %
@@ -86,7 +84,7 @@ static unsigned int
 %      reading.  A null image is returned if there is a memory shortage or
 %      if the image cannot be read.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -132,7 +130,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   IndexPacket
     index;
 
-  int
+  long
     y;
 
   PixelPacket
@@ -141,13 +139,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register IndexPacket
     *indexes;
 
-  register int
+  register long
     i,
     x;
 
   register PixelPacket
     *q;
 
+  ssize_t
+    count;
+
   TGAInfo
     tga_info;
 
@@ -157,31 +158,42 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     runlength;
 
   unsigned int
+    status;
+
+  unsigned long
     base,
     flag,
     offset,
     real,
-    skip,
-    status;
+    skip;
 
   /*
     Open image file.
   */
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenWarning,"Unable to open file",image);
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Read TGA header information.
   */
-  status=ReadBlob(image,1,(char *) &tga_info.id_length);
+  count=ReadBlob(image,1,&tga_info.id_length);
   tga_info.colormap_type=ReadBlobByte(image);
   tga_info.image_type=ReadBlobByte(image);
   do
   {
-    if ((status == False) || (tga_info.image_type == 0) ||
+    if ((count == 0) || (tga_info.image_type == 0) ||
         (tga_info.image_type > 11))
-      ThrowReaderException(CorruptImageWarning,"Not a TGA image file",image);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     tga_info.colormap_index=ReadBlobLSBShort(image);
     tga_info.colormap_length=ReadBlobLSBShort(image);
     tga_info.colormap_size=ReadBlobByte(image);
@@ -205,11 +217,6 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->storage_class=PseudoClass;
         image->colors=tga_info.colormap_length;
       }
-    if (image_info->ping)
-      {
-        CloseBlob(image);
-        return(image);
-      }
     if (tga_info.id_length != 0)
       {
         char
@@ -218,25 +225,24 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         /*
           TGA image comment.
         */
-        comment=(char *) AcquireMemory(tga_info.id_length+1);
+        comment=(char *) AcquireMagickMemory(tga_info.id_length+1);
         if (comment == (char *) NULL)
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
-        (void) ReadBlob(image,tga_info.id_length,comment);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        (void) ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
         comment[tga_info.id_length]='\0';
-        (void) SetImageAttribute(image,"Comment",comment);
-        LiberateMemory((void **) &comment);
+        (void) SetImageAttribute(image,"comment",comment);
+        comment=(char *) RelinquishMagickMemory(comment);
       }
-    GetPixelPacket(&pixel);
+    (void) ResetMagickMemory(&pixel,0,sizeof(PixelPacket));
+    pixel.opacity=TransparentOpacity;
     if (tga_info.colormap_type != 0)
       {
         /*
           Read TGA raster colormap.
         */
-        if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitWarning,"Memory allocation failed",
-            image);
-        for (i=0; i < (int) image->colors; i++)
+        if (AllocateImageColormap(image,image->colors) == False)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        for (i=0; i < (long) image->colors; i++)
         {
           switch (tga_info.colormap_size)
           {
@@ -246,7 +252,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              pixel.red=UpScale(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               pixel.green=pixel.red;
               pixel.blue=pixel.red;
               break;
@@ -259,10 +265,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               j=ReadBlobByte(image);
               k=ReadBlobByte(image);
-              pixel.red=(unsigned long) (MaxRGB*((int) (k & 0x7c) >> 2))/31;
-              pixel.green=(unsigned long)
-                (MaxRGB*(((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31;
-              pixel.blue=(unsigned long) (MaxRGB*((int) (j & 0x1f)))/31;
+              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.green=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+                ((int) (j & 0xe0) >> 5),31);
+              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
               break;
             }
             case 24:
@@ -271,15 +277,18 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue, green and red.
               */
-              pixel.blue=UpScale(ReadBlobByte(image));
-              pixel.green=UpScale(ReadBlobByte(image));
-              pixel.red=UpScale(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               break;
             }
           }
           image->colormap[i]=pixel;
         }
       }
+    if ((image_info->ping != False) && (image_info->number_scenes != 0))
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
+        break;
     /*
       Convert TGA pixels to pixel packets.
     */
@@ -290,16 +299,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     index=0;
     runlength=0;
     offset=0;
-    for (y=0; y < (int) image->rows; y++)
+    for (y=0; y < (long) image->rows; y++)
     {
       real=offset;
       if (((unsigned char) (tga_info.attributes & 0x20) >> 5) == 0)
         real=image->rows-real-1;
-      q=SetImagePixels(image,0,real,image->columns,1);
+      q=SetImagePixels(image,0,(long) real,image->columns,1);
       if (q == (PixelPacket *) NULL)
         break;
       indexes=GetIndexes(image);
-      for (x=0; x < (int) image->columns; x++)
+      for (x=0; x < (long) image->columns; x++)
       {
         if ((tga_info.image_type == TGARLEColormap) ||
             (tga_info.image_type == TGARLERGB) ||
@@ -312,17 +321,17 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               }
             else
               {
-                status=ReadBlob(image,1,(char *) &runlength);
-                if (status == False)
-                  ThrowReaderException(CorruptImageWarning,
-                    "Unable to read image data",image);
+                count=ReadBlob(image,1,&runlength);
+                if (count == 0)
+                  ThrowReaderException(CorruptImageError,
+                    "UnableToReadImageData");
                 flag=runlength & 0x80;
                 if (flag != 0)
                   runlength-=128;
                 skip=False;
               }
           }
-        if (!skip)
+        if (skip == False)
           switch (tga_info.bits_per_pixel)
           {
             case 8:
@@ -331,14 +340,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 Gray scale.
               */
-              index=ValidateColormapIndex(image,ReadBlobByte(image));
+              index=ReadBlobByte(image);
               if (tga_info.colormap_type != 0)
-                pixel=image->colormap[index];
+                pixel=image->colormap[ConstrainColormapIndex(image,index)];
               else
                 {
-                  pixel.red=(Quantum) UpScale(index);
-                  pixel.green=(Quantum) UpScale(index);
-                  pixel.blue=(Quantum) UpScale(index);
+                  pixel.red=ScaleCharToQuantum(index);
+                  pixel.green=ScaleCharToQuantum(index);
+                  pixel.blue=ScaleCharToQuantum(index);
                 }
               break;
             }
@@ -350,11 +359,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               */
               j=ReadBlobByte(image);
               k=ReadBlobByte(image);
-              pixel.red=(Quantum) ((MaxRGB*((int) (k & 0x7c) >> 2))/31);
-              pixel.green=(Quantum)
-                ((MaxRGB*(((int) (k & 0x03) << 3)+((int) (j & 0xe0) >> 5)))/31);
-              pixel.blue=(Quantum) ((MaxRGB*((int) (j & 0x1f)))/31);
-              index=ValidateColormapIndex(image,((unsigned short) k << 8)+j);
+              pixel.red=ScaleXToQuantum((int) (k & 0x7c) >> 2,31);
+              pixel.red=ScaleXToQuantum(((int) (k & 0x03) << 3)+
+                ((int) (j & 0xe0) >> 5),31);
+              pixel.blue=ScaleXToQuantum((int) (j & 0x1f),31);
+              if (image->storage_class == PseudoClass)
+                index=ConstrainColormapIndex(image,((unsigned short) k << 8)+j);
               break;
             }
             case 24:
@@ -363,17 +373,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               /*
                 8 bits each of blue green and red.
               */
-              pixel.blue=UpScale(ReadBlobByte(image));
-              pixel.green=UpScale(ReadBlobByte(image));
-              pixel.red=UpScale(ReadBlobByte(image));
+              pixel.blue=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.green=ScaleCharToQuantum(ReadBlobByte(image));
+              pixel.red=ScaleCharToQuantum(ReadBlobByte(image));
               if (tga_info.bits_per_pixel == 32)
-                pixel.opacity=MaxRGB-UpScale(ReadBlobByte(image));
+                pixel.opacity=ScaleCharToQuantum(ReadBlobByte(image));
               break;
             }
           }
         if (status == False)
-          ThrowReaderException(CorruptImageWarning,"Unable to read image data",
-            image);
+          ThrowReaderException(CorruptImageError,"UnableToReadImageData");
         if (image->storage_class == PseudoClass)
           indexes[x]=index;
         *q++=pixel;
@@ -390,24 +399,29 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           base++;
           offset=base;
         }
-      if (!SyncImagePixels(image))
-        break;
-      if (EOFBlob(image))
+      if (SyncImagePixels(image) == False)
         break;
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(LoadImageText,y,image->rows);
+          if (MagickMonitor(LoadImageTag,y,image->rows,exception) == False)
+            break;
     }
+    if (EOFBlob(image) != False)
+      {
+        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+          image->filename);
+        break;
+      }
     /*
       Proceed to next image.
     */
-    if (image_info->subrange != 0)
-      if (image->scene >= (image_info->subimage+image_info->subrange-1))
+    if (image_info->number_scenes != 0)
+      if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=ReadBlob(image,1,(char *) &tga_info.id_length);
+    count=ReadBlob(image,1,&tga_info.id_length);
     tga_info.colormap_type=ReadBlobByte(image);
     tga_info.image_type=ReadBlobByte(image);
-    status&=((tga_info.image_type != 0) && (tga_info.image_type <= 11));
+    status=((tga_info.image_type != 0) && (tga_info.image_type <= 11));
     if (status == True)
       {
         /*
@@ -416,17 +430,18 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AllocateNextImage(image_info,image);
         if (image->next == (Image *) NULL)
           {
-            DestroyImages(image);
+            DestroyImageList(image);
             return((Image *) NULL);
           }
-        image=image->next;
-        MagickMonitor(LoadImagesText,TellBlob(image),image->filesize);
+        image=SyncNextImageInList(image);
+        status=MagickMonitor(LoadImagesTag,TellBlob(image),GetBlobSize(image),
+          exception);
+        if (status == False)
+          break;
       }
   } while (status == True);
-  while (image->previous != (Image *) NULL)
-    image=image->previous;
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -440,7 +455,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterTGAImage adds attributes for the TGA image format to
+%  RegisterTGAImage() adds attributes for the TGA image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -458,29 +473,29 @@ ModuleExport void RegisterTGAImage(void)
     *entry;
 
   entry=SetMagickInfo("ICB");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
-  entry->description=AllocateString("Truevision Targa image");
-  entry->module=AllocateString("TGA");
-  RegisterMagickInfo(entry);
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
+  entry->description=AcquireString("Truevision Targa image");
+  entry->module=AcquireString("TGA");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
-  entry->description=AllocateString("Truevision Targa image");
-  entry->module=AllocateString("TGA");
-  RegisterMagickInfo(entry);
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
+  entry->description=AcquireString("Truevision Targa image");
+  entry->module=AcquireString("TGA");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
-  entry->description=AllocateString("Truevision Targa image");
-  entry->module=AllocateString("TGA");
-  RegisterMagickInfo(entry);
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
+  entry->description=AcquireString("Truevision Targa image");
+  entry->module=AcquireString("TGA");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
-  entry->decoder=ReadTGAImage;
-  entry->encoder=WriteTGAImage;
-  entry->description=AllocateString("Truevision Targa image");
-  entry->module=AllocateString("TGA");
-  RegisterMagickInfo(entry);
+  entry->decoder=(DecoderHandler *) ReadTGAImage;
+  entry->encoder=(EncoderHandler *) WriteTGAImage;
+  entry->description=AcquireString("Truevision Targa image");
+  entry->module=AcquireString("TGA");
+  (void) RegisterMagickInfo(entry);
 }
 
 /*
@@ -494,7 +509,7 @@ ModuleExport void RegisterTGAImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterTGAImage removes format registrations made by the
+%  UnregisterTGAImage() removes format registrations made by the
 %  TGA module from the list of supported formats.
 %
 %  The format of the UnregisterTGAImage method is:
@@ -504,10 +519,10 @@ ModuleExport void RegisterTGAImage(void)
 */
 ModuleExport void UnregisterTGAImage(void)
 {
-  UnregisterMagickInfo("ICB");
-  UnregisterMagickInfo("TGA");
-  UnregisterMagickInfo("VDA");
-  UnregisterMagickInfo("VST");
+  (void) UnregisterMagickInfo("ICB");
+  (void) UnregisterMagickInfo("TGA");
+  (void) UnregisterMagickInfo("VDA");
+  (void) UnregisterMagickInfo("VST");
 }
 
 /*
@@ -521,7 +536,7 @@ ModuleExport void UnregisterTGAImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteTGAImage writes a image in the Truevision Targa rasterfile
+%  WriteTGAImage() writes a image in the Truevision Targa rasterfile
 %  format.
 %
 %  The format of the WriteTGAImage method is:
@@ -534,9 +549,9 @@ ModuleExport void UnregisterTGAImage(void)
 %      False is returned is there is a memory shortage or if the image file
 %      fails to write.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
@@ -574,26 +589,33 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
       attributes;
   } TargaInfo;
 
-  ImageAttribute
+  const ImageAttribute
     *attribute;
 
-  int
-    count,
+  long
     y;
 
+  MagickOffsetType
+    scene;
+
+  register const PixelPacket
+    *p;
+
   register IndexPacket
     *indexes;
 
-  register int
-    i,
+  register long
     x;
 
-  register PixelPacket
-    *p;
+  register long
+    i;
 
   register unsigned char
     *q;
 
+  ssize_t
+    count;
+
   TargaInfo
     targa_info;
 
@@ -601,43 +623,53 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     *targa_pixels;
 
   unsigned int
-    scene,
     status;
 
   /*
     Open output image file.
   */
-  status=OpenBlob(image_info,image,WriteBinaryType);
+  assert(image_info != (const ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != False)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenWarning,"Unable to open file",image);
+    return(status);
   scene=0;
   do
   {
     /*
       Initialize TGA raster file header.
     */
-    TransformRGBImage(image,RGBColorspace);
+    (void) SetImageColorspace(image,RGBColorspace);
     targa_info.id_length=0;
-    attribute=GetImageAttribute(image,"Comment");
-    if (attribute != (ImageAttribute *) NULL)
-      targa_info.id_length=Min(Extent(attribute->value),255);
+    attribute=GetImageAttribute(image,"comment");
+    if (attribute != (const ImageAttribute *) NULL)
+      targa_info.id_length=Min(strlen(attribute->value),255);
     targa_info.colormap_type=0;
     targa_info.colormap_index=0;
     targa_info.colormap_length=0;
     targa_info.colormap_size=0;
     targa_info.x_origin=0;
     targa_info.y_origin=0;
-    targa_info.width=image->columns;
-    targa_info.height=image->rows;
+    targa_info.width=(unsigned short) image->columns;
+    targa_info.height=(unsigned short) image->rows;
     targa_info.bits_per_pixel=8;
     targa_info.attributes=0;
-    if (image->storage_class == DirectClass)
+    if ((image->storage_class == DirectClass) || (image->colors > 256))
       {
         /*
           Full color TGA raster.
         */
         targa_info.image_type=TargaRGB;
-        targa_info.bits_per_pixel=image->matte ? 32 : 24;
+        targa_info.bits_per_pixel=24;
+        if (image->matte != False)
+          {
+            targa_info.bits_per_pixel=32;
+            targa_info.attributes=8;  /* # of alpha bits */
+          }
       }
     else
       {
@@ -647,7 +679,7 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
         targa_info.image_type=TargaColormap;
         targa_info.colormap_type=1;
         targa_info.colormap_index=0;
-        targa_info.colormap_length=image->colors;
+        targa_info.colormap_length=(unsigned short) image->colors;
         targa_info.colormap_size=24;
       }
     /*
@@ -656,18 +688,19 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobByte(image,targa_info.id_length);
     (void) WriteBlobByte(image,targa_info.colormap_type);
     (void) WriteBlobByte(image,targa_info.image_type);
-    WriteBlobLSBShort(image,targa_info.colormap_index);
-    WriteBlobLSBShort(image,targa_info.colormap_length);
+    (void) WriteBlobLSBShort(image,targa_info.colormap_index);
+    (void) WriteBlobLSBShort(image,targa_info.colormap_length);
     (void) WriteBlobByte(image,targa_info.colormap_size);
-    WriteBlobLSBShort(image,targa_info.x_origin);
-    WriteBlobLSBShort(image,targa_info.y_origin);
-    WriteBlobLSBShort(image,targa_info.width);
-    WriteBlobLSBShort(image,targa_info.height);
+    (void) WriteBlobLSBShort(image,targa_info.x_origin);
+    (void) WriteBlobLSBShort(image,targa_info.y_origin);
+    (void) WriteBlobLSBShort(image,targa_info.width);
+    (void) WriteBlobLSBShort(image,targa_info.height);
     (void) WriteBlobByte(image,targa_info.bits_per_pixel);
     (void) WriteBlobByte(image,targa_info.attributes);
     if (targa_info.id_length != 0)
-      (void) WriteBlob(image,targa_info.id_length,attribute->value);
-    if (image->storage_class == PseudoClass)
+      (void) WriteBlob(image,targa_info.id_length,(unsigned char *)
+        attribute->value);
+    if (targa_info.image_type == TargaColormap)
       {
         unsigned char
           *targa_colormap;
@@ -676,67 +709,67 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           Dump colormap to file (blue, green, red byte order).
         */
         targa_colormap=(unsigned char *)
-          AcquireMemory(3*targa_info.colormap_length);
+          AcquireMagickMemory(3*targa_info.colormap_length);
         if (targa_colormap == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-            image);
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
         q=targa_colormap;
-        for (i=0; i < (int) image->colors; i++)
+        for (i=0; i < (long) image->colors; i++)
         {
-          *q++=DownScale(image->colormap[i].blue);
-          *q++=DownScale(image->colormap[i].green);
-          *q++=DownScale(image->colormap[i].red);
+          *q++=ScaleQuantumToChar(image->colormap[i].blue);
+          *q++=ScaleQuantumToChar(image->colormap[i].green);
+          *q++=ScaleQuantumToChar(image->colormap[i].red);
         }
-        (void) WriteBlob(image,(int) 3*targa_info.colormap_length,
-          (char *) targa_colormap);
-        LiberateMemory((void **) &targa_colormap);
+        (void) WriteBlob(image,3*targa_info.colormap_length,targa_colormap);
+        targa_colormap=(unsigned char *) RelinquishMagickMemory(targa_colormap);
       }
     /*
       Convert MIFF to TGA raster pixels.
     */
-    count=(unsigned int) (targa_info.bits_per_pixel*targa_info.width) >> 3;
-    targa_pixels=(unsigned char *) AcquireMemory(count);
+    count=(long) ((targa_info.bits_per_pixel*targa_info.width) >> 3);
+    targa_pixels=(unsigned char *) AcquireMagickMemory(count);
     if (targa_pixels == (unsigned char *) NULL)
-      ThrowWriterException(ResourceLimitWarning,"Memory allocation failed",
-        image);
-    for (y=(int) (image->rows-1); y >= 0; y--)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    for (y=(long) (image->rows-1); y >= 0; y--)
     {
-      p=GetImagePixels(image,0,y,image->columns,1);
-      if (p == (PixelPacket *) NULL)
+      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+      if (p == (const PixelPacket *) NULL)
         break;
       q=targa_pixels;
       indexes=GetIndexes(image);
-      for (x=0; x < (int) image->columns; x++)
+      for (x=0; x < (long) image->columns; x++)
       {
-        if (image->storage_class == PseudoClass)
+        if (targa_info.image_type == TargaColormap)
           *q++=indexes[x];
         else
           {
-            *q++=DownScale(p->blue);
-            *q++=DownScale(p->green);
-            *q++=DownScale(p->red);
+            *q++=ScaleQuantumToChar(p->blue);
+            *q++=ScaleQuantumToChar(p->green);
+            *q++=ScaleQuantumToChar(p->red);
+            if (image->matte != False)
+              *q++=ScaleQuantumToChar(p->opacity);
             if (image->colorspace == CMYKColorspace)
-              *q++=indexes[x];
-            else
-              if (image->matte)
-                *q++=MaxRGB-p->opacity;
+              *q++=ScaleQuantumToChar(indexes[x]);
           }
         p++;
       }
-      status=WriteBlob(image,q-targa_pixels,(char *) targa_pixels);
+      (void) WriteBlob(image,q-targa_pixels,targa_pixels);
       if (image->previous == (Image *) NULL)
         if (QuantumTick(y,image->rows))
-          MagickMonitor(SaveImageText,y,image->rows);
+          {
+            status=MagickMonitor(SaveImageTag,y,image->rows,&image->exception);
+            if (status == False)
+              break;
+          }
     }
-    LiberateMemory((void **) &targa_pixels);
+    targa_pixels=(unsigned char *) RelinquishMagickMemory(targa_pixels);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetNumberScenes(image));
-  } while (image_info->adjoin);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+    image=SyncNextImageInList(image);
+    status=MagickMonitor(SaveImagesTag,scene++,GetImageListLength(image),
+      &image->exception);
+    if (status == False)
+      break;
+  } while (image_info->adjoin != False);
   CloseBlob(image);
   return(True);
 }
